@@ -28,16 +28,27 @@ extern "C" {
 #endif
 
 #ifndef Py_BUILD_CORE /* not building the core - must be an ext */
-#    if defined(_MSC_VER)
-     /* So MSVC users need not specify the .lib file in
-      * their Makefile (other compilers are generally
-      * taken care of by distutils.) */
-#        ifdef _DEBUG
-#            error("debug first with cpython")    
-#            pragma comment(lib,"python27.lib")
-#        else
-#            pragma comment(lib,"python27.lib")
-#        endif /* _DEBUG */
+#  if defined(_MSC_VER) && !defined(_CFFI_)
+   /* So MSVC users need not specify the .lib file in
+    * their Makefile (other compilers are generally
+    * taken care of by distutils.) 
+    */
+#    ifdef _DEBUG
+#      error("debug first with cpython")    
+#      pragma comment(lib,"python27.lib")
+#    else
+#      pragma comment(lib,"python27.lib")
+#    endif /* _DEBUG */
+#    define HAVE_COPYSIGN 1
+#    define copysign _copysign
+#    ifdef MS_WIN64
+       typedef __int64 ssize_t;
+#    else
+       typedef _W64 int ssize_t;
+#    endif
+#define HAVE_SSIZE_T 1
+
+
 #    endif
 #endif /* _MSC_VER */
 
