@@ -87,6 +87,10 @@ PyAPI_FUNC(Signed) PyCode_GetNumFree(PyCodeObject *arg0);
 PyAPI_FUNC(PyCodeObject *) PyCode_New(int arg0, int arg1, int arg2, int arg3, PyObject *arg4, PyObject *arg5, PyObject *arg6, PyObject *arg7, PyObject *arg8, PyObject *arg9, PyObject *arg10, PyObject *arg11, int arg12, PyObject *arg13);
 #define PyCode_NewEmpty PyPyCode_NewEmpty
 PyAPI_FUNC(PyCodeObject *) PyCode_NewEmpty(const char *arg0, const char *arg1, int arg2);
+#define PyCodec_Decoder PyPyCodec_Decoder
+PyAPI_FUNC(PyObject *) PyCodec_Decoder(const char *arg0);
+#define PyCodec_Encoder PyPyCodec_Encoder
+PyAPI_FUNC(PyObject *) PyCodec_Encoder(const char *arg0);
 #define PyCodec_IncrementalDecoder PyPyCodec_IncrementalDecoder
 PyAPI_FUNC(PyObject *) PyCodec_IncrementalDecoder(const char *arg0, const char *arg1);
 #define PyCodec_IncrementalEncoder PyPyCodec_IncrementalEncoder
@@ -1035,6 +1039,10 @@ PyAPI_FUNC(void) _PyPyGC_AddMemoryPressure(Signed arg0);
 extern void _PyPy_Free(void *arg0);
 #define _PyPy_Malloc _PyPyPy_Malloc
 extern void * _PyPy_Malloc(Signed arg0);
+#define _PySet_Next _PyPySet_Next
+PyAPI_FUNC(int) _PySet_Next(PyObject *arg0, Signed *arg1, PyObject **arg2);
+#define _PySet_NextEntry _PyPySet_NextEntry
+PyAPI_FUNC(int) _PySet_NextEntry(PyObject *arg0, Signed *arg1, PyObject **arg2, Signed *arg3);
 #define _PyString_Eq _PyPyString_Eq
 PyAPI_FUNC(int) _PyString_Eq(PyObject *arg0, PyObject *arg1);
 #define _PyString_Join _PyPyString_Join
